@@ -130,6 +130,34 @@ FP::Lazy - lazy evaluation (delayed evaluation, promises)
     };
     like $@, qr/^Illegal division by zero/;
 
+    # A `lazyLight` promise is re-evaluated on every access:
+    my $z = 0;
+    my $v = lazyLight { $z++; 3*4 };
+    is force($v), 12;
+    is $z, 1;
+    is force($v), 12;
+    is $z, 2;
+
+    # There are 3 possible motivations for lazyLight: (1) lower
+    # allocation cost (save the wrapper data structure); (2) no risk
+    # for circular references (due to storing the result back into the
+    # wrapper (mutation) that can be used recursively); (3) to get
+    # fresh re-evaluation on every access and thus picking up any
+    # potential side effect.
+
+    # Arguably (3) is against the functional programming idea, and is
+    # a bit of a mis-use of lazyLight. For now, FP::TransparentLazy
+    # still helps this case by not using `FORCE` automatically. (TODO:
+    # provide another type that provides this with a guarantee?)
+
+    # Note that manual use of `FORCE` still stops the re-evalution:
+
+    ok ref $v;
+    is FORCE($v), 12;
+    is $z, 3;
+    is force($v), 12;
+    is $z, 3; # you can see that re-evaluation has stopped
+    ok not ref $v;
 
 =head1 DESCRIPTION
 
@@ -374,7 +402,7 @@ sub force {
     my ($perhaps_promise, $nocache) = @_;
 LP: {
         if (defined blessed($perhaps_promise)) {
-            if ($perhaps_promise->isa("FP::Lazy::PromiseLight")) {
+            if ($perhaps_promise->isa("FP::Lazy::PromiseLightBase")) {
                 $perhaps_promise = &$perhaps_promise;
                 redo LP;
             } elsif ($perhaps_promise->isa("FP::Lazy::Promise")) {
@@ -615,10 +643,11 @@ package FP::Lazy::Promise {
 
 my $lazyLight_thunk_show = subprefix_to_show_coderef("lazyLight ");
 
-package FP::Lazy::PromiseLight {
-    our @ISA = qw(FP::Lazy::AnyPromise);
+package FP::Lazy::PromiseLightBase {
 
-    use overload FP::Lazy::overloads(0);
+    # Things shared with FP::TransparentLazy::PromiseLight
+
+    our @ISA = qw(FP::Lazy::AnyPromise);
 
     sub FP_Lazy_is_forced {
         0
@@ -672,4 +701,9 @@ package FP::Lazy::PromiseLight {
     }
 }
 
+package FP::Lazy::PromiseLight {
+    our @ISA = qw(FP::Lazy::PromiseLightBase);
+    use overload FP::Lazy::overloads(0);
+}
+
 1
